@@ -10,7 +10,7 @@
 
 /************************************************************************
     FAUST Architecture File
-	Copyright (C) 2011 Michael J. Wilson
+    Copyright (C) 2011 Michael J. Wilson
 
     ---------------------------------------------------------------------
     This Architecture section is free software; you can redistribute
@@ -68,12 +68,15 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <iostream>
 #include <stack>
 #include <string>
 #include <map>
 #include <vector>
 #include <list>
 
+using namespace std;
+
 // On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
 // flags to avoid costly denormals
 #ifdef __SSE__
@@ -93,8 +96,6 @@ struct Meta : std::map<const char*, const char*>
     void declare (const char* key, const char* value) { (*this)[key]=value; }
 };
 
-#define max(x,y) (((x)>(y)) ? (x) : (y))
-#define min(x,y) (((x)<(y)) ? (x) : (y))
 
 #define sym(name) xsym(name)
 #define xsym(name) #name
@@ -207,7 +208,7 @@ DSSI_Program_Descriptor g_program_descriptor;
 std::vector<LADSPA_PortDescriptor> g_port_descriptors;
 std::vector<LADSPA_PortRangeHint> g_port_range_hints;
 std::vector<const char*> g_port_names;
-std::string g_name;
+const char* g_name;
 
 ////////////////////////////////////////////////////////////////////////////////
 // The enclosed code is from ladspa.cpp 
@@ -244,7 +245,8 @@ char* get_metadata_if_exists(const char* key, const char* default_string)
 {
     // TODO probably want to free these somehow.  Currently only used for ladspa descriptor
     Meta meta;
-    mydsp::metadata(&meta);
+    static mydsp dsp;
+    dsp.metadata(&meta);
     if(meta.find(key) != meta.end())
     {
         return strdup(meta[key]);
@@ -432,7 +434,7 @@ private:
     // Helper methods:
     void updateControlZones();
     void runImpl(unsigned long, snd_seq_event_t *, unsigned long);
-    void addSamples(int);
+    void addSamples(size_t);
 
     // Helpers for UI building:
     // TODO organize this a bit better later...
@@ -842,10 +844,10 @@ void Plugin::runImpl(unsigned long sampleCount, snd_seq_event_t *events, unsigne
     }
 }
 
-void Plugin::addSamples(int samples)
+void Plugin::addSamples(size_t samples)
 {
     size_t i;
-    int j;
+    size_t j;
     size_t v;
 
     // TODO this isn't very efficient right now...
@@ -1185,9 +1187,9 @@ extern "C"
             // Fill the descriptors:
             // TODO figure out strdup with const strings
             g_ladspa_descriptor->UniqueID = 0;
-            g_ladspa_descriptor->Label = get_metadata_if_exists("name", g_name.c_str());
+            g_ladspa_descriptor->Label = get_metadata_if_exists("name", g_name);
             g_ladspa_descriptor->Properties = LADSPA_PROPERTY_HARD_RT_CAPABLE;
-            g_ladspa_descriptor->Name = get_metadata_if_exists("name", g_name.c_str());
+            g_ladspa_descriptor->Name = get_metadata_if_exists("name", g_name);
             g_ladspa_descriptor->Maker = get_metadata_if_exists("author", "Maker");
             g_ladspa_descriptor->Copyright = get_metadata_if_exists("copyright", "Copyright");
             g_ladspa_descriptor->PortCount = g_port_descriptors.size();
@@ -1218,7 +1220,7 @@ extern "C"
             // Program description (TODO if we eventually support multiple programs we will need to handle this differently)
             g_program_descriptor.Bank = 0;
             g_program_descriptor.Program = 0;
-            g_program_descriptor.Name = get_metadata_if_exists("name", g_name.c_str());
+            g_program_descriptor.Name = get_metadata_if_exists("name", g_name);
 
             delete temp_mydsp;
             delete temp_descriptor_ui;
