@@ -19,20 +19,19 @@
  ************************************************************************
  ************************************************************************/
 
-
-
 #include <stdio.h>
 #include <string.h>
-#include "doc_Text.hh"
-#include "compatibility.hh"
 #include <string>
 #include <vector>
 #include <iostream>
 #include <sstream>
-#include <assert.h>
 #include <cmath>
+#include <stdlib.h>
 
 #include "floats.hh"
+#include "doc_Text.hh"
+#include "compatibility.hh"
+#include "exception.hh"
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
@@ -50,11 +49,8 @@
 #define M_E 2.71828182845904523536
 #endif
 
-extern bool gInternDoubleSwitch;
 const string symbolicNumber (double n);
 
-
-
 #if 0
 /**
  * Suppress trailing zero in a string representating a floating point number.
@@ -141,9 +137,9 @@ static bool AlmostEqual(double A, double B)
  */
 bool isPiPower (double n, string& s)
 {
-    assert(n>0);
+    faustassert(n>0);
     stringstream ss (stringstream::out|stringstream::in);
-    int k = floor(log(n)/log(M_PI));
+    int k = (int)floor(log(n)/log(M_PI));
     if ( AlmostEqual(n, exp(k * log(M_PI))) && (k!=0) && (abs(k)<5.0) ) {
         ss << "\\pi";
         if (k!=1)  ss << "^{"<< k <<"}";
@@ -161,10 +157,10 @@ bool isPiPower (double n, string& s)
  */
 bool isExpPower (double n, string& s)
 {
-    assert(n>0);
+    faustassert(n>0);
     stringstream ss (stringstream::out|stringstream::in);
-    int k = floor(log(n));
-    if ( AlmostEqual(n, exp(k)) && (k!=0) && (abs(k)<5.0) ) {
+    int k = (int)floor(log(n));
+    if ( AlmostEqual(n, exp(float(k))) && (k!=0) && (abs(k)<5.0) ) {
         ss << "e";
         if (k!=1)  ss << "^{"<< k <<"}";
         s = ss.str();
@@ -181,7 +177,7 @@ bool isExpPower (double n, string& s)
  */
 bool isSymbolicPower (double n, string& s)
 {
-    assert(n>0);
+    faustassert(n>0);
     if (isPiPower(n,s)) {
         return true;
     } else if (isExpPower(n,s)) {
@@ -218,7 +214,7 @@ const string addFraction (int num, int denom, const string& exp)
 const string positiveSymbolicNumber (double n)
 {
     string s;
-    assert(n>0);
+    faustassert(n>0);
 
     // Try to find a symbolic representation
 
