@@ -19,16 +19,17 @@
  ************************************************************************
  ************************************************************************/
 
-
-
 // SVGDev.cpp
 
 #include "SVGDev.h"
-#include "stdio.h"
-#include <iostream>
-using namespace std;
+#include "global.hh"
+#include "exception.hh"
 
-extern bool gShadowBlur;
+#include <stdio.h>
+#include <iostream>
+#include <sstream>
+
+using namespace std;
 
 static char* xmlcode(const char* name, char* name2)
 {
@@ -51,31 +52,34 @@ static char* xmlcode(const char* name, char* name2)
 	return name2;
 }
 
-SVGDev::SVGDev(const char* ficName,double largeur, double hauteur)
+SVGDev::SVGDev(const char* ficName, double largeur, double hauteur)
 {
 	double gScale = 0.5;
-//	if ((fic_repr = fopen(addFileNum(ficName),"w+")) == NULL) {
 	if ((fic_repr = fopen(ficName,"w+")) == NULL) {
-	//if((fic_repr = fopen(ficName,"w+")) == NULL) {
-		cout<<"Impossible de creer ou d'ouvrir "<<ficName<<endl;
+        stringstream error;
+		error << "Impossible to create or open " << ficName << endl;
+        throw faustexception(error.str());
 	}
 
 	// representation file:
 	fprintf(fic_repr,"<?xml version=\"1.0\"?>\n");
 	// + DTD ...
 	// viewBox:
-	//fprintf(fic_repr,"<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" preserveAspectRatio=\"xMidYMid meet\" viewBox=\"0 0 %f %f\" width=\"200mm\" height=\"150mm\" >\n",largeur,hauteur);
-	fprintf(fic_repr,"<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" viewBox=\"0 0 %f %f\" width=\"%fmm\" height=\"%fmm\" version=\"1.1\">\n", largeur, hauteur, largeur*gScale, hauteur*gScale);
+    if (gGlobal->gScaledSVG) {
+        fprintf(fic_repr,"<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" viewBox=\"0 0 %f %f\" width=\"100%%\" height=\"100%%\" version=\"1.1\">\n", largeur, hauteur);
+    } else {
+        fprintf(fic_repr,"<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" viewBox=\"0 0 %f %f\" width=\"%fmm\" height=\"%fmm\" version=\"1.1\">\n", largeur, hauteur, largeur*gScale, hauteur*gScale);
+    }
 
-    if (gShadowBlur) {
-     fprintf(fic_repr,
-        "<defs>\n"
-        "   <filter id=\"filter\" filterRes=\"50\" x=\"0\" y=\"0\">\n"
-        "     <feGaussianBlur in=\"SourceGraphic\" stdDeviation=\"1.5\"/>\n"
-        "     <feOffset dx=\"1\" dy=\"1\"/>\n"
-        "   </filter>\n"
-        "</defs>\n"
-        );
+    if (gGlobal->gShadowBlur) {
+		 fprintf(fic_repr,
+		    "<defs>\n"
+		    "   <filter id=\"filter\" filterRes=\"18\" x=\"0\" y=\"0\">\n"
+		    "     <feGaussianBlur in=\"SourceGraphic\" stdDeviation=\"1.55\" result=\"blur\"/>\n"
+		    "     <feOffset in=\"blur\" dx=\"3\" dy=\"3\"/>\n"
+		    "   </filter>\n"
+		    "</defs>\n"
+		    );
     }
 
 }
@@ -88,29 +92,54 @@ SVGDev::~SVGDev()
 
 void SVGDev::rect(double x,double y,double l,double h, const char* color, const char* link)
 {
-	char buf[512];
-	if (link != 0 && link[0]!=0) {
-		// open the optional link tag
-		fprintf(fic_repr,"<a xlink:href=\"%s\">\n", xmlcode(link, buf));
-	}
-	// draw the shadow
-    if (gShadowBlur) {
-        fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"stroke:none;fill:#888888;;filter:url(#filter);\"/>\n",x+1,y+1,l,h);
+    char buf[512];
+    if (link != 0 && link[0]!=0) {
+        // open the optional link tag
+        fprintf(fic_repr,"<a xlink:href=\"%s\">\n", xmlcode(link, buf));
+    }
+    // draw the shadow
+    if (gGlobal->gShadowBlur) {
+        fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0.1\" ry=\"0.1\" style=\"stroke:none;fill:#aaaaaa;;filter:url(#filter);\"/>\n",x+1,y+1,l,h);
     } else {
         fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"stroke:none;fill:#cccccc;\"/>\n",x+1,y+1,l,h);
     }
 
-	// draw the rectangle
-	//fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"shape-rendering: crispEdges; stroke: black;stroke-width:0.25;fill:%s;\"/>\n", x, y, l, h, color);
-	//fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"stroke: black;stroke-width:0.25;fill:%s;\"/>\n", x, y, l, h, color);
-	fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"stroke:none;fill:%s;\"/>\n", x, y, l, h, color);
-	if (link != 0 && link[0]!=0) {
-		// close the optional link tag
-		fprintf(fic_repr,"</a>\n");
-	}
+    // draw the rectangle
+    fprintf(fic_repr,"<rect x=\"%f\" y=\"%f\" width=\"%f\" height=\"%f\" rx=\"0\" ry=\"0\" style=\"stroke:none;fill:%s;\"/>\n", x, y, l, h, color);
+    if (link != 0 && link[0]!=0) {
+        // close the optional link tag
+        fprintf(fic_repr,"</a>\n");
+    }
 
 }
 
+//<polygon fill="lightsteelblue" stroke="midnightblue" stroke-width="5"
+//    points="350,180 380,180 380,160 410,160 410,180 440,180 440,140 470,140 470,180
+//    500,180 500,120 530,120 530,180" />
+
+void SVGDev::triangle(double x,double y,double l,double h, const char* color, const char* link, bool leftright)
+{
+    char buf[512];
+    if (link != 0 && link[0]!=0) {
+        // open the optional link tag
+        fprintf(fic_repr,"<a xlink:href=\"%s\">\n", xmlcode(link, buf));
+    }
+    // draw triangle+circle
+    float r = 1.5; // circle radius
+    float x0, x1, x2;
+    if (leftright) {
+        x0 = (float)x;
+        x1 = (float)(x+l-2*r);
+        x2 = (float)(x+l-r);
+    } else {
+        x0 = (float)(x+l);
+        x1 = (float)(x+2*r);
+        x2 = (float)(x+r);
+    }
+    fprintf(fic_repr,"<polygon fill=\"%s\" stroke=\"black\" stroke-width=\".25\" points=\"%f,%f %f,%f %f,%f\"/>\n", color, x0,y, x1,y+h/2.0, x0,y+h);
+    fprintf(fic_repr,"<circle  fill=\"%s\" stroke=\"black\" stroke-width=\".25\" cx=\"%f\" cy=\"%f\" r=\"%f\"/>\n", color, x2, y+h/2.0, r);
+}
+
 void SVGDev::rond(double x,double y,double rayon)
 {
 	fprintf(fic_repr,"<circle cx=\"%f\" cy=\"%f\" r=\"%f\"/>\n",x,y,rayon);
@@ -140,7 +169,6 @@ void SVGDev::carre(double x,double y,double cote)
 
 void SVGDev::trait(double x1,double y1,double x2,double y2)
 {
-	//fprintf(fic_repr,"<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\"  style=\"shape-rendering: crispEdges; stroke:black; stroke-linecap:round; stroke-width:0.25;\"/>\n",x1,y1,x2,y2);
 	fprintf(fic_repr,"<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\"  style=\"stroke:black; stroke-linecap:round; stroke-width:0.25;\"/>\n",x1,y1,x2,y2);
 }
 
@@ -149,19 +177,25 @@ void SVGDev::dasharray(double x1,double y1,double x2,double y2)
 	fprintf(fic_repr,"<line x1=\"%f\" y1=\"%f\" x2=\"%f\" y2=\"%f\"  style=\"stroke: black; stroke-linecap:round; stroke-width:0.25; stroke-dasharray:3,3;\"/>\n",x1,y1,x2,y2);
 }
 
-void SVGDev::text(double x,double y,const char* name)
+void SVGDev::text(double x,double y,const char* name, const char* link)
 {
-	char name2[256];
-	//fprintf(fic_repr,"<text x=\"%f\" y=\"%f\" font-family=\"Courier\" font-size=\"8\" text-anchor=\"middle\">%s</text>\n",x,y+3,xmlcode(name,name2));
+    char buf[512];
+    if (link != 0 && link[0]!=0) {
+        // open the optional link tag
+        fprintf(fic_repr,"<a xlink:href=\"%s\">\n", xmlcode(link, buf));
+    }
+    char name2[256];
 	fprintf(fic_repr,"<text x=\"%f\" y=\"%f\" font-family=\"Arial\" font-size=\"7\" text-anchor=\"middle\" fill=\"#FFFFFF\">%s</text>\n",x,y+2,xmlcode(name,name2));
+    if (link != 0 && link[0]!=0) {
+        // close the optional link tag
+        fprintf(fic_repr,"</a>\n");
+    }
 }
 
 void SVGDev::label(double x,double y,const char* name)
 {
 	char name2[256];
-//	fprintf(fic_repr,"<text x=\"%f\" y=\"%f\" style=\"font-family:Courier;font-weight:normal;font-style:normal;font-size:7\">%s</text>\n",x,y+2,xmlcode(name,name2));
 	fprintf(fic_repr,"<text x=\"%f\" y=\"%f\" font-family=\"Arial\" font-size=\"7\">%s</text>\n",x,y+2,xmlcode(name,name2));
-//	fprintf(fic_repr,"<text x=\"%f\" y=\"%f\" font-family=\"Courier\" font-size=\"7\" fill=\"#000000\">%s</text>\n",x,y+2,xmlcode(name,name2));
 }
 
 void SVGDev::markSens(double x,double y,int sens)
