@@ -19,8 +19,6 @@
  ************************************************************************
  ************************************************************************/
  
- 
- 
 // SVGDev.h
 
 #if !defined SVGDEV_H
@@ -33,16 +31,17 @@ class SVGDev : public device
 	public:
 		SVGDev(const char*,double,double);
 		~SVGDev();
-		void rect(double,double,double,double, const char*, const char*);
-		void rond(double,double,double);
-		void carre(double,double,double);
-		void fleche(double,double,double,int);
-		void trait(double,double,double,double);
-		void dasharray(double,double,double,double);
-		void text(double,double,const char*);
-		void label(double,double,const char*);
-		void markSens(double,double,int);
-		void Error(const char*,const char*,int,double,double,double);
+        void rect(double x,double y,double l,double h, const char* color, const char* link);
+        void triangle(double x,double y,double l,double h, const char* color, const char* link, bool leftright);
+        void rond(double x,double y,double rayon);
+        void carre(double x,double y,double cote);
+        void fleche(double x,double y,double rotation,int sens);
+        void trait(double x1,double y1,double x2,double y2);
+        void dasharray(double x1,double y1,double x2,double y2);
+        void text(double x,double y,const char* name, const char* link);
+        void label(double x,double y,const char* name);
+        void markSens(double x,double y,int sens);
+        void Error(const char* message, const char* reason,int nb_error,double x,double y,double largeur);
 
 };
 
