@@ -18,29 +18,24 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
- 
- 
- 
+
 #include "propagate.hh"
 #include "prim2.hh"
-#include <assert.h>
 #include "ppbox.hh"
 #include "xtended.hh"
 #include "labels.hh"
 #include "Text.hh"
 #include "ppsig.hh"
 #include "names.hh"
-
-//extern bool gPrintDocSwitch;
-//static siglist realPropagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig);
-
+#include "exception.hh"
+#include "global.hh"
 
 ////////////////////////////////////////////////////////////////////////
 /**
  * propagate : box listOfSignal-> listOfSignal'
  *
- * Propage une liste de signaux de l'entrée vers la sortie d'une boite
- * La boite a été annotée aec son type 
+ * Propage a list of signals into a box expression representing a
+ * signal processor
  */
 ///////////////////////////////////////////////////////////////////////
 
@@ -48,7 +43,7 @@
 //! mix une liste de signaux sur n bus				
 siglist mix(const siglist& lsig, int nbus)
 {
-	int nlines	= lsig.size();
+	int nlines	= (int)lsig.size();
 	
 	siglist dst(nbus);
 	
@@ -65,7 +60,7 @@ siglist mix(const siglist& lsig, int nbus)
 //! split une liste de signaux sur n bus				
 siglist split(const siglist& inputs, int nbus)
 {
-	int nlines	= inputs.size();
+	int nlines	= (int)inputs.size();
 	
 	siglist outputs(nbus);
 	
@@ -116,8 +111,8 @@ siglist listRange(const siglist& l, int i, int j)
 
 siglist listConcat(const siglist& a, const siglist& b)
 {
-	int n1 = a.size();
-	int n2 = b.size();
+	int n1 = (int)a.size();
+	int n2 = (int)b.size();
 	siglist r(n1+n2);
 	
 	for (int x=0; x<n1; x++) r[x] = a[x];
@@ -130,30 +125,31 @@ siglist listConcat(const siglist& a, const siglist& b)
  */
 Tree listConvert(const siglist& a)
 {
-	int 	n = a.size();
-	Tree 	t=nil;
+	int 	n = (int)a.size();
+	Tree 	t=gGlobal->nil;
+
 	while (n--) t = cons(a[n],t);
 	return t;
 }
 
-// siglist listConvertBack(Tree l)
-// {
-// 	siglist r;
-// 	while (!isNil(l)) { r.push_back(hd(l)); l = tl(l); }
-// 	return r;
-// }
+/**
+ * Convert a tree list of signals into an stl list of signals
+ */
+ void treelist2siglist(Tree l, siglist& r)
+ {
+    r.clear();
+    while (!isNil(l)) { r.push_back(hd(l)); l = tl(l); }
+ }
 
 siglist listLift(const siglist& l)
 {
-	int 		n = l.size();
+	int 		n = (int)l.size();
 	siglist		r(n);
 	
 	for(int i = 0; i<n; i++) r[i] = lift(l[i]);
 	return r;
 }
 
-static int	gDummyInput = 10000;
-
 /**
  * Propagate computes the outputs signals of a block-diagram according to a list of input signals.
  *
@@ -163,40 +159,89 @@ static int	gDummyInput = 10000;
  *\param lsig list of signals to be propagated into box
  *\return list of resulting signals
  */
-/*
-// for debugging purposes
-
-siglist realpropagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig);
-
-siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
-{
-	cerr << "propagate in " << boxpp(box) << endl; 
-	for (int i=0; i<lsig.size(); i++) { cerr << " -> signal " << i << " : " << *(lsig[i]) << endl; }
-	cerr << endl;
-	return realpropagate (slotenv, path, box, lsig);
-}
-*/
 
 /**
- * Old try for names propagation.
+ * Store the propagation result as a property of the arguments tuplet
+ * @param args propagation arguments
+ * @param value propagation result
  */
-//siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
-//{
-//	siglist S = realPropagate(slotenv, path, box, lsig);
-//
-//	if (gPrintDocSwitch) {
-//		Tree	id;
-//		if (lsig.size()==0 && getDefNameProperty(box, id)) {
-//			string nickname = defName2NickName(tree2str(id));
-//			//setSigListNickName(S, nickname);
-//		}
-//	}
-//
-//	return S;
-//}
+void setPropagateProperty(Tree args, const siglist&  lsig)
+{
+    setProperty(args, tree(gGlobal->PROPAGATEPROPERTY), listConvert(lsig));
+}
 
-//siglist realPropagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
-siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
+
+/**
+ * Retreive the propagation result as a property of the arguments tuplet
+ * @param args propagation arguments
+ * @param lsig the propagation result if any
+ * @return true if a propagation result was stored
+ */
+bool getPropagateProperty(Tree args, siglist&  lsig)
+{
+    Tree value;
+    if (getProperty(args, tree(gGlobal->PROPAGATEPROPERTY), value)) {
+        treelist2siglist(value, lsig);
+        return true;
+    } else {
+        return false;
+    }
+}
+
+
+/**
+ * Propagate a list of signals into a block diagram.
+ * @param slotenv environment associating slots and signals
+ * @param path user interface group path
+ * @param box the block diagram
+ * @param lsig the list of input signals to propagate
+ * @return the resulting list of output signals
+ */
+
+siglist realPropagate (Tree slotenv, Tree path, Tree box, const siglist& lsig);
+
+
+/**
+ * Propagate a list of signals into a block diagram. Do memoization.
+ * @param slotenv environment associating slots and signals
+ * @param path user interface group path
+ * @param box the block diagram
+ * @param lsig the list of input signals to propagate
+ * @return the resulting list of output signals
+ */
+
+siglist propagate (Tree slotenv, Tree path, Tree box, const siglist& lsig)
+{
+    Tree args = tree(gGlobal->PROPAGATEPROPERTY,slotenv,path,box,listConvert(lsig));
+    siglist result;
+    if (! getPropagateProperty(args, result)) {
+        result = realPropagate (slotenv, path, box, lsig);
+        setPropagateProperty(args, result);
+    }
+    //cerr << "propagate in " << boxpp(box) << endl;
+    //for (int i=0; i<lsig.size(); i++) { cerr << " -> signal " << i << " : " << *(lsig[i]) << endl; }
+    //cerr << endl;
+    return result;
+}
+
+// Apply sigFTZ() to all signals of a vector
+static siglist wrapWithFTZ(const siglist&  l1) 
+{
+	siglist l2;
+	for (auto x : l1) { l2.push_back(sigFTZ(x)); }
+	return l2;
+}
+
+/**
+ * Propagate a list of signals into a block diagram. Actual function.
+ * @param slotenv environment associating slots and signals
+ * @param path user interface group path
+ * @param box the block diagram
+ * @param lsig the list of input signals to propagate
+ * @return the resulting list of output signals
+ */
+
+siglist realPropagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 {
 	int		i;
 	double	r;
@@ -207,7 +252,8 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 	prim4	p4;
 	prim5	p5;
 	
-	Tree	t1, t2, ff, label, cur, min, max, step, type, name, file, slot, body;
+    Tree	t1, t2, ff, label, cur, min, max, step, type, name, file, slot, body, chan;
+    tvec    wf;
 	
 	
 	xtended* xt = (xtended*)getUserData(box);
@@ -215,40 +261,48 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 	// Extended Primitives
 	
 	if (xt)	{
-		assert(lsig.size() == xt->arity());
+		faustassert(lsig.size() == xt->arity());
 		return makeList(xt->computeSigOutput(lsig));
 	}
 		
 	// Numbers and Constants
 	
 	else if (isBoxInt(box, &i)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigInt(i)); 
 	}
 	else if (isBoxReal(box, &r)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigReal(r)); 
 	}
-    
+
+    // A Waveform has two outputs it size and a period signal representing its content
+
+    else if (isBoxWaveform(box)) 	{
+        faustassert(lsig.size()==0);
+        const tvec br = box->branches();
+        return listConcat(makeList(sigInt(br.size())), makeList(sigWaveform(br)));
+    }
+
     else if (isBoxFConst(box, type, name, file))    { 
-        assert(lsig.size()==0); 
+        faustassert(lsig.size()==0); 
         return makeList(sigFConst(type, name, file)); 
     }
     
     else if (isBoxFVar(box, type, name, file))    { 
-        assert(lsig.size()==0); 
+        faustassert(lsig.size()==0); 
         return makeList(sigFVar(type, name, file)); 
     }
 	
 	// Wire and Cut
 	
 	else if (isBoxCut(box)) 				{ 
-		assert(lsig.size()==1); 
+		faustassert(lsig.size()==1); 
 		return siglist(); 
 	}
 	
 	else if (isBoxWire(box)) 				{ 
-		assert(lsig.size()==1); 
+		faustassert(lsig.size()==1); 
 		return lsig;  
 	}
 	
@@ -256,97 +310,132 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 	
 	else if (isBoxSlot(box)) 				{ 
 		Tree sig;
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		if (!searchEnv(box,sig,slotenv)) {
 			// test YO simplification des diagrames
 			//fprintf(stderr, "propagate : internal error (slot undefined)\n");
-			//exit(1);
-			sig = sigInput(++gDummyInput);
+			sig = sigInput(++gGlobal->gDummyInput);
 		}
 		return makeList(sig);
 	}
 	
 	else if (isBoxSymbolic(box, slot, body)) 				{ 
-		assert(lsig.size()>0); 
-		return propagate(pushEnv(slot,lsig[0],slotenv), path, body, listRange(lsig, 1, lsig.size()));
+		faustassert(lsig.size()>0); 
+		return propagate(pushEnv(slot,lsig[0],slotenv), path, body, listRange(lsig, 1, (int)lsig.size()));
 	}
 	
 	// Primitives
 	
 	else if (isBoxPrim0(box, &p0)) 			{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList( p0() );  
 	}
 	
 	else if (isBoxPrim1(box, &p1)) 				{ 
-		assert(lsig.size()==1); 
+		faustassert(lsig.size()==1); 
 		return makeList( p1(lsig[0]) );  
 	}
 	
 	else if (isBoxPrim2(box, &p2)) 				{ 
 //		printf("prim2 recoit : "); print(lsig); printf("\n");
-		assert(lsig.size()==2); 
-		return makeList( p2(lsig[0],lsig[1]) );  
+		faustassert(lsig.size()==2);
+        if (p2 == &sigEnable) {
+            if (gGlobal->gEnableFlag) {
+                // special case for sigEnable that requires a transformation
+                // enable(X,Y) -> sigEnable(X*Y, Y>0)
+                return makeList( sigEnable( sigMul(lsig[0],lsig[1]), sigGT(lsig[1],sigReal(0.0)) ) );
+            } else {
+                // We gEnableFlag is false we replace enable by a simple multiplication
+                return makeList( sigMul(lsig[0],lsig[1]) );
+            }
+        } else if (p2 == &sigControl) {
+            if (gGlobal->gEnableFlag) {
+                // special case for sigEnable that requires a transformation
+                // enable(X,Y) -> sigEnable(X*Y, Y>0)
+                return makeList( sigEnable( lsig[0], lsig[1] ) );
+            } else {
+                // We gEnableFlag is false we replace control by identity function
+                return makeList( lsig[0] );
+            }
+        }
+        return makeList( p2(lsig[0],lsig[1]) );
 	}
 	
 	else if (isBoxPrim3(box, &p3)) 				{ 
-		assert(lsig.size()==3); 
+		faustassert(lsig.size()==3); 
 		return makeList( p3(lsig[0],lsig[1],lsig[2]) );  
 	}
 	
 	else if (isBoxPrim4(box, &p4)) 				{ 
-		assert(lsig.size()==4); 
+		faustassert(lsig.size()==4); 
 		return makeList( p4(lsig[0],lsig[1],lsig[2],lsig[3]) );  
 	}
 	
 	else if (isBoxPrim5(box, &p5)) 				{ 
-		assert(lsig.size()==5); 
+		faustassert(lsig.size()==5); 
 		return makeList( p5(lsig[0],lsig[1],lsig[2],lsig[3],lsig[4]) );  
 	}
 	
 	else if (isBoxFFun(box, ff)) 				{ 
 		//cerr << "propagate en boxFFun of arity " << ffarity(ff) << endl;
-		assert(int(lsig.size())==ffarity(ff)); 
+		faustassert(int(lsig.size())==ffarity(ff)); 
 		return makeList(sigFFun(ff, listConvert(lsig)));  
 	}
 	
 	// User Interface Widgets
 	
 	else if (isBoxButton(box, label)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigButton(normalizePath(cons(label, path)))); 
 	}
 	
 	else if (isBoxCheckbox(box, label)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigCheckbox(normalizePath(cons(label, path)))); 
 	}
 	
 	else if (isBoxVSlider(box, label, cur, min, max, step)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigVSlider(normalizePath(cons(label, path)), cur, min, max, step)); 
 	}
 	
 	else if (isBoxHSlider(box, label, cur, min, max, step)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigHSlider(normalizePath(cons(label, path)), cur, min, max, step)); 
 	}
-	
+
 	else if (isBoxNumEntry(box, label, cur, min, max, step)) 	{ 
-		assert(lsig.size()==0); 
+		faustassert(lsig.size()==0); 
 		return makeList(sigNumEntry(normalizePath(cons(label, path)), cur, min, max, step)); 
 	}
 	
 	else if (isBoxVBargraph(box, label, min, max)) 	{ 
-		assert(lsig.size()==1); 
+		faustassert(lsig.size()==1); 
 		return makeList(sigVBargraph(normalizePath(cons(label, path)), min, max, lsig[0])); 
 	}
 	
 	else if (isBoxHBargraph(box, label, min, max)) 	{ 
-		assert(lsig.size()==1); 
+		faustassert(lsig.size()==1); 
 		return makeList(sigHBargraph(normalizePath(cons(label, path)), min, max, lsig[0])); 
 	}
 	
+	else if (isBoxSoundfile(box, label, chan)) 	{ 
+		faustassert(lsig.size()==1);
+		Tree fullpath = normalizePath(cons(label, path));
+		Tree soundfile = sigSoundfile(fullpath);
+		int c = tree2int(chan);
+		siglist lsig2(c+2);
+		lsig2[0] = sigSoundfileLength(soundfile);
+		lsig2[1] = sigSoundfileRate(soundfile);
+
+		// compute bound limited read index : max(0, min(ridx,length-1))
+		Tree ridx = tree(gGlobal->gMaxPrim->symbol(), sigInt(0), tree(gGlobal->gMinPrim->symbol(), lsig[0], sigAdd(lsig2[0],sigInt(-1))));
+		for (int i = 0; i<c; i++) {
+			lsig2[i+2] = sigSoundfileChannel(soundfile, sigInt(i), ridx);
+		}
+		return lsig2; 
+	}
+	
 	// User Interface Groups
 	
 	else if (isBoxVGroup(box, label, t1)) 	{ 
@@ -368,7 +457,7 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 		getBoxType(t1, &in1, &out1);
 		getBoxType(t2, &in2, &out2);
 
-        assert(out1==in2);
+        faustassert(out1==in2);
 
 		if (out1 == in2) {
 			return propagate(slotenv, path, t2, propagate(slotenv, path,t1,lsig));
@@ -408,19 +497,7 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
 		siglist l2 = mix(l1, in2);
 		return propagate(slotenv, path, t2, l2);
 	}
-/*	
-	else if (isBoxRec(box, t1, t2)) 	{ 
-		int in1, out1, in2, out2;
-		getBoxType(t1, &in1, &out1);
-		getBoxType(t2, &in2, &out2);
-		
-		siglist l0 = makeSigProjList(ref(1), in2);
-		siglist l1 = propagate(slotenv, path, t2, l0);
-		siglist l2 = propagate(slotenv, path, t1, listConcat(l1,listLift(lsig)));
-		Tree g = rec(listConvert(l2));
-		return makeSigProjList(g, out1);
-	}
-*/	
+
     else if (isBoxRec(box, t1, t2)) 	{
         // Bug Corrected
         int in1, out1, in2, out2;
@@ -432,18 +509,28 @@ siglist propagate (Tree slotenv, Tree path, Tree box, const siglist&  lsig)
         siglist l0 = makeMemSigProjList(ref(1), in2);
         siglist l1 = propagate(slotenv2, path, t2, l0);
         siglist l2 = propagate(slotenv2, path, t1, listConcat(l1,listLift(lsig)));
-        Tree g = rec(listConvert(l2));
+		siglist l3 = (gGlobal->gFTZMode > 0) ? wrapWithFTZ(l2) : l2;
+        Tree g = rec(listConvert(l3));
         return makeSigProjList(g, out1);
     }
 
-	cout << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", unrecognised box expression : " << boxpp(box) << endl;
-	exit(1);
+    stringstream error;
+    error << "ERROR in file " << __FILE__ << ':' << __LINE__ << ", unrecognised box expression : " << boxpp(box) << endl;
+    throw faustexception(error.str());
+
 	return siglist();
 }
 
-	
+/**
+ * Top level propagate a list of signals into a block diagram. Do memoization.
+ * @param path user interface group path
+ * @param box the block diagram
+ * @param lsig the list of input signals to propagate
+ * @return the resulting list of output signals
+ */
+
 Tree boxPropagateSig (Tree path, Tree box, const siglist& lsig)
 {
-	return listConvert(propagate(nil, path, box, lsig));
+	return listConvert(propagate(gGlobal->nil, path, box, lsig));
 }
 
