@@ -18,11 +18,20 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
-#include <assert.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <limits.h>
+#include <set>
+
 #include "recursivness.hh"
+#include "property.hh"
+#include "exception.hh"
+#include "global.hh"
+#include "signals.hh"
+#include "ppsig.hh"
+
+using namespace std;
 
 /**
  * @file recursivness.cpp
@@ -37,12 +46,9 @@
 
 //--------------------------------------------------------------------------
 static int annotate(Tree env, Tree sig);
-static int position (Tree env, Tree t, int p=1);
-
-Tree RECURSIVNESS = tree(symbol("RecursivnessProp"));
+static int position (Tree env, Tree t, int p = 1);
 //--------------------------------------------------------------------------
 
-
 /**
  * Annotate a signal with recursivness. Should be used before
  * calling getRecursivness
@@ -50,10 +56,9 @@ Tree RECURSIVNESS = tree(symbol("RecursivnessProp"));
  */
 void recursivnessAnnotation(Tree sig)
 {
-	annotate(nil, sig);
+	annotate(gGlobal->nil, sig);
 }
 
-
 /**
  * Return the recursivness of a previously
  * annotated signal. An error is generated
@@ -64,9 +69,10 @@ void recursivnessAnnotation(Tree sig)
 int getRecursivness(Tree sig)
 {
 	Tree tr;
-	if ( ! getProperty(sig, RECURSIVNESS, tr)) {
-		cerr << "Error in getRecursivness of " << *sig << endl;
-		exit(1);
+	if (!getProperty(sig, gGlobal->RECURSIVNESS, tr)) {
+        stringstream error;
+        error << "ERROR in getRecursivness of " << *sig << endl;
+        throw faustexception(error.str());
 	}
 	return tree2int(tr);
 }
@@ -82,7 +88,7 @@ static int annotate(Tree env, Tree sig)
 {
 	Tree tr, var, body;
 
-	if (getProperty(sig, RECURSIVNESS, tr)) {
+	if (getProperty(sig, gGlobal->RECURSIVNESS, tr)) {
 		return tree2int(tr);	// already annotated
 	} else if (isRec(sig, var, body)) {
 		int p = position(env, sig);
@@ -90,18 +96,18 @@ static int annotate(Tree env, Tree sig)
 			return p;	// we are inside \x.(...)
 		} else {
 			int r = annotate(cons(sig, env), body) - 1;
-			if (r<0) r=0;
-			setProperty(sig, RECURSIVNESS, tree(r));
+			if (r<0) r = 0;
+			setProperty(sig, gGlobal->RECURSIVNESS, tree(r));
 			return r;
 		}
 	} else {
 		int rmax = 0;
 		vector<Tree> v; getSubSignals(sig, v);
-		for (unsigned int i=0; i<v.size(); i++) {
+		for (unsigned int i = 0; i < v.size(); i++) {
 			int r = annotate(env, v[i]);
-			if (r>rmax) rmax=r;
+			if (r>rmax) rmax = r;
 		}
-		setProperty(sig, RECURSIVNESS, tree(rmax));
+		setProperty(sig, gGlobal->RECURSIVNESS, tree(rmax));
 		return rmax;
 	}
 }
@@ -114,9 +120,59 @@ static int annotate(Tree env, Tree sig)
  * @param t signal we want to know the position
  * @return the position in the recursive environment
  */
-static int position (Tree env, Tree t, int p)
+static int position(Tree env, Tree t, int p)
 {
 	if (isNil(env)) return 0;	// was not in the environment
 	if (hd(env) == t) return p;
 	else return position (tl(env), t, p+1);
 }
+
+//-----------------------------------list recursive symbols-----------------------
+
+/**
+ * return the set of recursive symbols appearing in a signal.
+ * @param sig the signal to analyze
+ * @return the set of symbols
+ */
+
+Tree symlistVisit(Tree sig, set<Tree>& visited)
+{
+    Tree S;
+    
+    if (gGlobal->gSymListProp->get(sig, S)) {
+        return S;
+    } else if (visited.count(sig) > 0) {
+        return gGlobal->nil;
+    } else {
+        visited.insert(sig);
+        Tree id, body;
+        if (isRec(sig, id, body)) {
+            Tree U = singleton(sig);
+            for (int i = 0; i < len(body); i++) {
+                U = setUnion(U, symlistVisit(nth(body,i), visited));
+            }
+            return U;
+        } else {
+            vector<Tree> subsigs;
+            int n = getSubSignals(sig, subsigs, true); // il faut visiter aussi les tables
+            Tree U = gGlobal->nil;
+            for (int i = 0; i < n; i++) {
+                U = setUnion(U, symlistVisit(subsigs[i], visited));
+            }
+            return U;
+        }
+    }
+}
+
+Tree symlist(Tree sig)
+{
+    Tree S;
+    
+    if (!gGlobal->gSymListProp->get(sig, S)) {
+        set<Tree> visited;
+        S = symlistVisit(sig, visited);
+        gGlobal->gSymListProp->set(sig, S);
+    }
+    //cerr << "SYMLIST " << *S << " OF " << ppsig(sig) << endl;
+    return S;
+}
